/**********************************************************************
 *
 * browseaction.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: browseaction.cpp 213 1999-03-29 02:14:31Z sjboddie $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.2  1999/03/29 02:14:27  sjboddie

   More changes to browseaction

   Revision 1.1  1999/03/25 03:09:40  sjboddie

   subjectbrowseaction became browseaction


 */


#include "browseaction.h"
#include "browsetools.h"

browseaction::browseaction () {
  // this action uses cgi variables "a", "d", "cl",
  // "x", "gc", and "gt"
  cgiarginfo arg_ainfo;
  arg_ainfo.shortname = "a";
  arg_ainfo.longname = "action";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "p";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (NULL, arg_ainfo);

  arg_ainfo.shortname = "d";
  arg_ainfo.longname = "document OID";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::none;
  arg_ainfo.argdefault = "";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);

  arg_ainfo.shortname = "cl";
  arg_ainfo.longname = "classification OID";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::none;
  arg_ainfo.argdefault = "";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);

  // in this action "gc" controls the expand/contract
  // contents function
  arg_ainfo.shortname = "gc";
  arg_ainfo.longname = "expand contents";
  arg_ainfo.multiplechar = false;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "0";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);

  // in this action "gt" controls the expand/contract
  // text function
  arg_ainfo.shortname = "gt";
  arg_ainfo.longname = "expand text";
  arg_ainfo.multiplechar = false;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "0";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);

  // "x" is 0 normally or 1 if page
  // has been "detached"
  arg_ainfo.shortname = "x";
  arg_ainfo.longname = "detached page";
  arg_ainfo.multiplechar = false;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "0";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);
}

browseaction::~browseaction () {
}

bool browseaction::check_cgiargs (cgiargsclass &/*args*/, ostream &/*logout*/) {
  // don't want to check anything yet.
  return true;
}

void browseaction::get_cgihead_info (cgiargsclass &/*args*/, response_t &response,
				     text_t &response_data, ostream &/*logout*/) {
  response = content;
  response_data = "text/html";
}


// define all the macros which are related to pages generated
// by this action
void browseaction::define_internal_macros (displayclass &disp, cgiargsclass &args, 
					   recptproto *collectproto, ostream &logout) {

  text_t ctype, cname, toc, navbar;

  collectproto->get_classification_info (args["cl"], ctype, cname);

  // set the _toc_ macro
  if (ctype == "hierarchy") get_hierarchy_toc (args, collectproto, toc, logout);
  else if (ctype == "book") get_book_toc (args, collectproto, toc, logout);
  else if (ctype == "datelist") {
    get_datelist_toc (args, collectproto, toc, logout);
    set_arrow_links (disp, args, collectproto, cname, logout);
    set_classification_links (disp, args, collectproto, cname, logout);
  } else {
    get_list_toc (args, collectproto, toc, logout); // list is the default
    set_arrow_links (disp, args, collectproto, cname, logout);
    set_classification_links (disp, args, collectproto, cname, logout);
  }
  disp.setmacro ("toc", cname, toc);

  // set the _navigationbar_ macro
  set_navigation_bar (disp, args, collectproto, cname, logout);
}


bool browseaction::do_action (cgiargsclass &args, recptproto *collectproto,
			      displayclass &disp, outconvertclass &outconvert, 
			      ostream &textout, ostream &/*logout*/) {

  if (collectproto == NULL) return false;

  text_t ctype, cname;
  collectproto->get_classification_info (args["cl"], ctype, cname);

  textout << outconvert << disp << ("_" + cname + ":header_\n")
	  << ("_" + cname + ":content_\n") << ("_" + cname + ":footer_\n");

  return true;
}

